@@ -1,89 +1,15 @@
 package Mojo::JWT::Google;
-use utf8;
-use Mojo::Base qw(Mojo::JWT);
-use Mojo::Collection 'c';
-use Mojo::File ();
+use parent Mojo::JWT;
+use strictures;
+use vars qw($VERSION);
+use Time::HiRes qw/gettimeofday/;
+use Mojo::Util qw(slurp);
 use Mojo::JSON qw(decode_json);
-use Carp;
 
-our $VERSION = '0.09';
-
-has client_email => undef;
-has expires_in   => 3600;
-has issue_at     => undef;
-has scopes       => sub { c() };
-has target       => q(https://www.googleapis.com/oauth2/v4/token);
-has user_as      => undef;
-has audience     => undef;
-
-sub new {
-  my ($class, %options) = @_;
-  my $self = $class->SUPER::new(%options);
-  return $self if not defined $self->{from_json};
-
-  my $result = $self->from_json($self->{from_json});
-
-  if ( $result == 0 ) {
-    croak 'Your JSON file import failed.';
-    return undef;
-  }
-  return $self;
+BEGIN {
+  $Mojo::JWT::Google::VERSION = '0.01';
 }
-
-#Mojo::JWT::Google->attr( claims => sub {
-#  return shift->_construct_claims;
-#}, sub { {} });
-
-sub claims {
-  my ($self, $value) = @_;
-  if (defined $value) {
-    $self->{claims} = $value;
-    return $self;
-  }
-  my $claims = $self->_construct_claims;
-  unless (exists $claims->{exp}) {
-    $claims->{exp} = $self->now + $self->expires_in ;
-  }
-  return $claims;
-}
-
-sub _construct_claims {
-  my $self = shift;
-  my $result = {};
-  $result->{iss}   = $self->client_email;
-  $result->{aud}   = $self->target;
-  $result->{sub}   = $self->user_as if defined $self->user_as;
-  my @scopes = @{ $self->scopes };
-
-  croak "Can't use both scopes and audience in the same token" if @scopes && $self->audience;
-  $result->{scope} = c(@scopes)->join(' ')->to_string if @scopes;
-  $result->{target_audience} = $self->audience if defined $self->audience;
-
-  if ( not defined $self->issue_at ) {
-    $self->set_iat(1);
-  }
-  else {
-    $self->set_iat(0);
-    $result->{iat} = $self->issue_at;
-    $result->{exp} = $self->issue_at + $self->expires_in;
-  }
-  return $result;
-}
-
-sub from_json {
-  my ($self, $value) = @_;
-  return 0 if not defined $value;
-  return 0 if not -f $value;
-  my $json = decode_json( Mojo::File->new($value)->slurp );
-  return 0 if not defined $json->{private_key};
-  return 0 if $json->{type} ne 'service_account';
-  $self->algorithm('RS256');
-  $self->secret($json->{private_key});
-  $self->client_email($json->{client_email});
-  return 1
-}
-
-1;
+my $grant_type = "urn:ietf:params:oauth:grant-type:jwt-bearer";
 
 
 =head1 NAME
@@ -92,13 +18,13 @@ Mojo::JWT::Google - Service Account tokens
 
 =head1 VERSION
 
-0.09
+0.01
 
 =head1 SYNOPSIS
 
-  my $gjwt = Mojo::JWT::Google->new(secret => 's3cr3t',
-                                    scopes => [ '/my/scope/a', '/my/scope/b' ],
-                                    client_email => 'riche@cpan.org')->encode;
+my $gjwt = Mojo::JWT::Google->new(secret => 's3cr3t',
+                                  scopes => [ '/my/scope/a', '/my/scope/b' ],
+                                  client_email => 'riche@cpan.org')->encode;
 
 =head1 DESCRIPTION
 
@@ -120,12 +46,23 @@ somewhere.  This will ease some busy work in configuring the object -- with
 virtually the only things to do is determine the scopes and the user_as if you
 need to impersonate.
 
-  my $gjwt = Mojo::JWT::Google
-    ->new( from_json => '/my/secret.json',
-           scopes    => [ '/my/scope/a', '/my/scope/b' ])->encode;
+my $gjwt = Mojo::JWT::Google
+  ->new( from_json => '/my/secret.json',
+         scopes    => [ '/my/scope/a', '/my/scope/b' ])->encode;
+
 
 =cut
 
+
+
+sub new {
+  my ($class, %options) = @_;
+  my $self = bless \%options, $class;
+  $self->{scopes} = [] if not defined $self->{scopes};
+  $self->from_json($self->{from_json}) if defined $self->{from_json};
+  return $self;
+}
+
 =head1 ATTRIBUTES
 
 L<Mojo::JWT::Google> inherits all attributes from L<Mojo::JWT> and defines the
@@ -136,31 +73,27 @@ following new ones.
 Overrides the parent class and constructs a hashref representing Google's
 required attribution.
 
+=cut
 
-=head2 client_email
+sub claims {
+  my ($self) = @_;
 
-Get or set the Client ID email address.
+  $self->issue_at( defined $self->issue_at   ? $self->issue_at :
+                                               (gettimeofday)[0] );
 
-=head2 expires_in
+  $self->expire_at( defined $self->expire_at ? $self->expire_at :
+                                               $self->issue_at + 3600 );
 
-Defines the threshold for when the token expires.  Defaults to 3600.
-
-=head2 issue_at
-
-Defines the time of issuance in epoch seconds. If not defined, the claims issue
-at date defaults to the time when it is being encoded.
-
-=head2 scopes
-
-Get or set the Google scopes.  If impersonating, these scopes must be set up by
-your Google Business Administrator.
-
-=head2 target
-
-Get or set the target.  At the time of writing, there is only one valid target:
-https://www.googleapis.com/oauth2/v4/token.  This is the default value; if you
-have no need to customize this, then just fetch the default.
+  my $result = { iss   => $self->client_email,
+                 scope => join( ' ', @{ $self->scopes } ),
+                 aud   => $self->target,
+                 exp   => $self->expire_at,
+                 iat   => $self->issue_at,
+               };
 
+  $result->{sub} = $self->user_as if defined $self->user_as;
+  return $result;
+}
 
 =head2 user_as
 
@@ -170,6 +103,79 @@ successfully.
 
 =cut
 
+sub user_as {
+  my ($self, $value) = @_;
+  $self->{user_as} = $value if defined $value;
+  return $self->{user_as};
+}
+
+=head2 scopes
+
+Get or set the Google scopes.  If impersonating, these scopes must be set up by
+your Google Business Administrator.
+
+=cut
+
+sub scopes {
+  my ($self, $value) = @_;
+  push @{ $self->{scopes} }, $value if defined $value;
+  return $self->{scopes};
+}
+
+=head2 client_email
+
+Get or set the Client ID email address.
+
+=cut
+
+sub client_email {
+  my ($self, $value) = @_;
+  $self->{client_email} = $value if defined $value;
+  return $self->{client_email};
+}
+
+=head2 target
+
+Get or set the target.  At the time of writing, there is only one valid target:
+https://www.googleapis.com/oauth2/v3/token.  This is the default value; if you
+have no need to customize this, then just fetch the default.
+
+=cut
+
+sub target {
+  my ($self, $value) = @_;
+  $self->{target} = $value if defined $value;
+  $self->{target} = q(https://www.googleapis.com/oauth2/v3/token)
+    if not defined $self->{target};
+  return $self->{target};
+}
+
+=head2 expire_at
+
+Defines when the token expires.  The maximum is 60 minutes from the issue
+epoch time.  The default is 60 minutes from the issue epoch time.
+
+=cut
+
+sub expire_at {
+  my ($self, $value) = @_;
+  $self->{expire_at} = $value if defined $value;
+  return $self->{expire_at};
+}
+
+=head2 issue_at
+
+Defines the time of issuance in epoch seconds. If not defined, the claims issue
+at date defaults to the time when it is being encoded.
+
+=cut
+
+sub issue_at {
+  my ($self, $value) = @_;
+  $self->{issue_at} = $value if defined $value;
+  return $self->{issue_at};
+}
+
 =head1 METHODS
 
 Inherits all methods from L<Mojo::JWT> and defines the following new ones.
@@ -183,24 +189,33 @@ Returns 0 on failure: file not found or value not defined
 
  $gjwt->from_json('/my/google/app/project/sa/json/file');
 
+=cut
+
+sub from_json {
+  my ($self, $value) = @_;
+  return 0 if not defined $value;
+  return 0 if not -f $value;
+  my $json = decode_json( slurp ( $value ) );
+  return 0 if not defined $json->{private_key};
+  return 0 if $json->{type} ne 'service_account';
+  $self->secret($json->{private_key});
+  $self->client_email($json->{client_email});
+}
+
+1;
 
 =head1 SEE ALSO
 
 L<Mojo::JWT>
 
-=head1 SOURCE REPOSITORY
+SOURCE REPOSITORY
 
-L<http://github.com/rabbiveesh/Mojo-JWT-Google>
+L<http://github.com/rpcme/Mojo-JWT-Google>
 
 =head1 AUTHOR
 
 Richard Elberger, <riche@cpan.org>
 
-=head1 CONTRIBUTORS
-
-Scott Wiersdorf, <scott@perlcode.org>
-Avishai Goldman, <veesh@cpan.org>
-
 =head1 COPYRIGHT AND LICENSE
 
 Copyright (C) 2015 by Richard Elberger
